@@ -35,15 +35,13 @@
 #define _HAVE_XALLOC_DECLS
 #include "ephyrlog.h"
 #include <GL/glxproto.h>
-#include "GL/glx/glxserver.h"
-#include "GL/glx/indirect_table.h"
-#include "GL/glx/indirect_util.h"
-#include "GL/glx/unpack.h"
+#include "glx/glxserver.h"
+#include "glx/indirect_table.h"
+#include "glx/indirect_util.h"
+#include "glx/unpack.h"
 #include "hostx.h"
 
 
-#ifdef XEPHYR_DRI
-
 #ifndef TRUE
 #define TRUE 1
 #endif
@@ -245,10 +243,9 @@ ephyrGLXGetVisualConfigsReal (__GLXclientState *a_cl,
 
 out:
     EPHYR_LOG ("leave\n") ;
-    if (props_buf) {
-        xfree (props_buf) ;
-        props_buf = NULL ;
-    }
+    xfree (props_buf) ;
+    props_buf = NULL ;
+
     return res ;
 }
 
@@ -303,10 +300,9 @@ ephyrGLXGetFBConfigsSGIXReal (__GLXclientState *a_cl,
 
 out:
     EPHYR_LOG ("leave\n") ;
-    if (props_buf) {
-        xfree (props_buf) ;
-        props_buf = NULL ;
-    }
+    xfree (props_buf) ;
+    props_buf = NULL ;
+
     return res ;
 }
 
@@ -393,14 +389,12 @@ ephyrGLXQueryServerString(__GLXclientState *a_cl, GLbyte *a_pc)
 
 out:
     EPHYR_LOG ("leave\n") ;
-    if (server_string) {
-        xfree (server_string) ;
-        server_string = NULL;
-    }
-    if (buf) {
-        xfree (buf);
-        buf = NULL;
-    }
+    xfree (server_string) ;
+    server_string = NULL;
+
+    xfree (buf);
+    buf = NULL;
+
     return res ;
 }
 
@@ -727,6 +721,3 @@ ephyrGLXIsDirectSwap (__GLXclientState *a_cl, GLbyte *a_pc)
 {
     return ephyrGLXIsDirectReal (a_cl, a_pc, TRUE) ;
 }
-
-#endif /*XEPHYR_DRI*/
-
